//-------+---------+---------+---------+---------+---------+---------+-----
// wingmidi.c - Multi-Media Wingman Ver01.01 Eternal 
//            - Wingman Standard MIDI File FM Sound Player
//-------+---------+---------+---------+---------+---------+---------+-----

///////////////////////////////////////////////////////////////////////////
// Copyright (C) GENERAL SCIENCE (M.Kasahara) since 1999.01
// Programmed by GENERAL SCIENCE (M.Kasahara)
// Ver00.40 2000.09 - Wingman Ver01.01
///////////////////////////////////////////////////////////////////////////
// !!! Information !!! - Find ">>>" for find function in this source.
///////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <WGM/mwingman.h>

#include "resource.h"

static char g_cCommand [32];
static char g_cFileName [100][80];
static int g_nFiles = 0;
static int g_nPerc = PERCUSSION;
static Bool g_bFM4OP = FALSE;
static int g_nReverb = 0;
static int g_nChorus = 0;

//-------+---------+---------+---------+---------+---------+---------+-----
// Define Local Prototype :
//-------+---------+---------+---------+---------+---------+---------+-----
static void arginit (int argc, char *argv []);
static void usage (int nUsage);
static void OutSynthInfo (void);
static void OutMIDIInfo (wgmMIDITrack **pTrack, int nDivision);

//-------+---------+---------+---------+---------+---------+---------+-----
// Main function :
//-------+---------+---------+---------+---------+---------+---------+-----
// Function List >>> main :
//               >>> arginit :
//               >>> usage :
//               >>> OutSynthInfo :
//               >>> OutMIDIInfo :

///////////////////////////////////////////////////////////////////////////
//>>> main :
///////////////////////////////////////////////////////////////////////////
int main (int argc, char *argv [])
{
  wgmMIDITrack **pTrack;
  int nTracks, nDivision;
  int i, nDevice;

  arginit (argc, argv);
  if (!g_nFiles)
  {
    usage (TRUE);
    exit (1);
  }
  usage (FALSE);

  if ((nDevice = wgmOpenDevice_Midi (g_nPerc, g_bFM4OP, g_nReverb, g_nChorus))
     < 0)
    exit (1);
  OutSynthInfo ();

  for (i=0; i<g_nFiles; i++)
  {

    pTrack = wgmRead_Midi (g_cFileName [i], &nTracks, &nDivision);
    if (pTrack != NULL)
    {
      printf ("----------------------------\n");
      printf ("File name   : %s\n", g_cFileName [i]);
      printf ("Tracks      : %d\n", nTracks);
      printf ("Division    : %d\n", nDivision);
      OutMIDIInfo (pTrack, nDivision);

      if (wgmPlay_Midi (pTrack, nTracks, nDivision) == FALSE)
        fprintf (stderr, "Error: Can't play midi\n");

      wgmEnd_Midi ();
      wgmFreeMIDITrack_Midi (pTrack);
    }

  }

  wgmCloseDevice_Midi ();

  exit (0);
}

///////////////////////////////////////////////////////////////////////////
//>>> arginit :
///////////////////////////////////////////////////////////////////////////
void arginit (int argc, char *argv [])
{
  char *lpstrArgv0;
  int i, nFile = 0;
  int nPerc = 0;

  if ((lpstrArgv0 = strrchr (argv [0], '/')) != NULL)
    lpstrArgv0++;
  else
    lpstrArgv0 = argv [0];
  strcpy (g_cCommand, lpstrArgv0);

  for (i=1; i<argc; i++)
  {
    if ((strcmp (argv [i], "-perc") == 0)
      || (strcmp (argv [i], "-p") == 0))
    {
      if (i < argc - 1)
      {
        if (0 < atoi (argv [i+1]) && atoi (argv [i+1]) < 17)
        {
          if (!nPerc)
            g_nPerc = 0;
          g_nPerc |= 1 << (atoi (argv [i+1]) - 1);
          nPerc++;
        }
        i++;
      }
    }else if ((strcmp (argv [i], "-reverb") == 0)
      || (strcmp (argv [i], "-r") == 0)){
      if (i < argc - 1)
      {
        if (0<atoi (argv [i+1]) && atoi (argv [i+1])<128)
          g_nReverb = atoi (argv [i+1]);
        i++;
      }
    }else if ((strcmp (argv [i], "-chorus") == 0)
      || (strcmp (argv [i], "-c") == 0)){
      if (i < argc - 1)
      {
        if (0 < atoi (argv [i+1]) && atoi (argv [i+1]) < 128)
          g_nChorus = atoi (argv [i+1]);
        i++;
      }
    }else if ((strcmp (argv [i], "-op4") == 0)
      || (strcmp (argv [i], "-o") == 0)){
      g_bFM4OP = TRUE;
    }else if ((strcmp (argv [i], "-help") == 0)
      || (strcmp (argv [i], "-h") == 0)){
      usage (TRUE);
      exit (0);
    }else{
      if (g_nFiles < 100)
      {
        strcpy (g_cFileName [g_nFiles], argv [i]);
        g_nFiles++;
      }else{
        fprintf (stderr, "Warring: Too many file. Less than 100 files.\n");
        return;
      }
    }
  }
}

///////////////////////////////////////////////////////////////////////////
//>>> usage :
///////////////////////////////////////////////////////////////////////////
void usage (int nUsage)
{
  fprintf (stderr, ">>> " APPNAME " <<<\n");
  fprintf (stderr, "- " APPVERSION " " APPDATE " -\n");
  fprintf (stderr, "- Copyright (C) " APPPRODUCT " -\n");
  fprintf (stderr, "\n");
  fprintf (stderr, " Wingman Standard MIDI File FM Sound Player "
                   "(Command Line Version) ... \n");
  fprintf (stderr, "\n");
  if (nUsage == FALSE)
    return;

  fprintf (stderr, " Usage: %s [-op4] [-perc <channel>] [-reverb <value>]",
    g_cCommand);
  fprintf (stderr, " [-chorus <value>]\n");
  fprintf (stderr, "         [-help] MidiFiles\n");

  fprintf (stderr,
    " -op4(or -o)            ... Enable FM4OP sound.\n");
  fprintf (stderr,
    " -perc(or -p) <channel> ... Set percussion channel (1-16)\n");
  fprintf (stderr,
    " -reverb(or -r) <value> ... Set reverb value (0-127)\n");
  fprintf (stderr,
    " -chorus(or -c) <value> ... Set chorus value (0-127)\n");
  fprintf (stderr,
    " -help(or -h)           ... This message\n");
  fprintf (stderr,
    "  MidiFiles             ... Play Standard midi files (*.mid)\n\n");
}

///////////////////////////////////////////////////////////////////////////
//>>> OutSynthInfo :
///////////////////////////////////////////////////////////////////////////
void OutSynthInfo (void)
{
  struct synth_info **pSynthInfo;
  int i, j, nSynths;

  nSynths = wgmGetSynths_Midi ();
  if (nSynths<=0)
    return;

  pSynthInfo = wgmGetSynthInfo_Midi (nSynths);

  if (pSynthInfo != NULL)
  {
    printf ("----------Card Info---------\n");
    for (i=0; i<nSynths; i++)
    {
      printf ("No          : %d\n", i+1);
      printf ("Card Name   : %s\n", pSynthInfo [i]->name);
      printf ("Device      : %d\n", pSynthInfo [i]->device);
      printf ("Type        : %d\n", pSynthInfo [i]->synth_type);
      printf ("Sub type    : %d\n", pSynthInfo [i]->synth_subtype);
      printf ("Perc mode   : %d\n", pSynthInfo [i]->perc_mode);
      printf ("Voices      : %d\n", pSynthInfo [i]->nr_voices);
      printf ("Drums       : %d\n", pSynthInfo [i]->nr_drums);
      printf ("Bank size   : %d\n", pSynthInfo [i]->instr_bank_size);
      printf ("Capabilities: %d\n\n", pSynthInfo [i]->capabilities);
    }
    wgmFreeSynthInfo_Midi (pSynthInfo);
  }

  return;
}

///////////////////////////////////////////////////////////////////////////
//>>> OutMIDIInfo :
///////////////////////////////////////////////////////////////////////////
void OutMIDIInfo (wgmMIDITrack **pTrack, int nDivision)
{
  wgmMIDISequence *pSeq;
  int i, j, nTimes = 0;

  if (pTrack == NULL)
    return;

  printf ("-----Times & Track Info-----\n");
  for (i=0; pTrack [i] != 0; i++)
  {
    for (j=0; j<pTrack [i]->nEvents; j++)
    {
      pSeq = pTrack [i]->pSeq [j];
      if (pSeq->nTime == 0)
      {
        if (0<pSeq->nCmd && pSeq->nCmd<8)
          printf ("Track%02d     : %s\n", i+1, pSeq->pText);
      }else{
        break;
      }
    }
  }
  nTimes = wgmGetTimes_Midi (pTrack, nDivision); 
  printf ("Total Times : %d'%02d\"%02d\n\n",
    nTimes/100/60, (nTimes/100)%60, nTimes%100);

  return;
}

